from Products.validation.interfaces import ivalidator

class VacationsValidator:

    __implements__ = (ivalidator,)

    def __init__(self, name):
        self.name = name

# username, year, month, day, hours, day_portion, reason, red, green, blue
    def __call__(self, value, *args, **kwargs):
        self.lines = {'username'            : [],
                      'realname'            : [],
                      'groupname'           : [],
                      'year'                : [],
                      'month'               : [],
                      'day'                 : [],
                      'day_portion'         : [],
                      'reason'              : [],
                      'red'                 : [],
                      'green'               : [],
                      'blue'                : []}
                  
        # setup and check for item count errors:

        line_errors = []
        line_count = 1
        for vacation_string in value:
            username = ""
            realname = ""
            groupname = ""
            year = ""
            month = ""
            day = ""
            day_portion = ""
            reason = ""
            red = ""
            green = ""
            blue = ""
            try:
                items = eval('[%s]' % vacation_string)
            except:
                return "Mixed numeric and text input somewhere on line %d" % line_count
            if len(items) != 11:
                line_errors.append("%d" % line_count)
            else:
                username, realname, groupname, year, month, day, day_portion, reason, red, green, blue = items 
                self.lines['username'].append(username)
                self.lines['realname'].append(realname)
                self.lines['groupname'].append(groupname)
                self.lines['year'].append(year)
                self.lines['month'].append(month)
                self.lines['day'].append(day)
                self.lines['day_portion'].append(day_portion)
                self.lines['reason'].append(reason)
                self.lines['red'].append(red)
                self.lines['green'].append(green)
                self.lines['blue'].append(blue)
            line_count += 1
        if len(line_errors) > 0:
            return "Wrong number of items on lines: %s" % line_errors

        # check for basic conversion errors:

        error_message = ""
        errors = None

        errors = self.checkForUsernameErrors()
        if errors:
            error_message = "username (field 1) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForRealNameErrors()
        if errors:
            error_message = "realname (field 2) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForGroupNameErrors()
        if errors:
            error_message = "groupname (field 3) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForYearErrors()
        if errors:
            error_message = "year (field 4) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForMonthErrors()
        if errors:
            error_message = "month (field 5) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForDayErrors()
        if errors:
            error_message = "day (field 6) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForDayPortionErrors()
        if errors:
            error_message = "day_portion (field 7) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForReasonErrors()
        if errors:
            error_message = "reason (field 8) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForRedErrors()
        if errors:
            error_message = "Red (field 9) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForGreenErrors()
        if errors:
            error_message = "Green (field 10) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForBlueErrors()
        if errors:
            error_message = "Blue (field 11) error on lines: %s" % str(errors)
            return error_message

        # check for start and end time range errors:
        error_message = ""

        # check for color range errors:
        errors = self.checkForColorRangeErrors()
        if errors:
            error_message = "One or more color values out of range (0-255) on lines: %s" % str(errors)
            return error_message

        return 1

    def checkForYearErrors(self):
        line_errors = []
        count = 1
        for year in self.lines['year']:
            try:
                int(year)
            except:
                line_errors.append(count)
            if int(year) < 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForMonthErrors(self):
        line_errors = []
        count = 1
        for month in self.lines['month']:
            try:
                int(month)
            except:
                line_errors.append(count)
            if int(month) < 0 or int(month) > 12:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForDayErrors(self):
        line_errors = []
        count = 1
        for day in self.lines['day']:
            try:
                int(day)
            except:
                line_errors.append(count)
            if int(day) < 0 or int(day) > 31:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForUsernameErrors(self):
        # pass
        return None

    def checkForRealNameErrors(self):
        # pass
        return None

    def checkForGroupNameErrors(self):
        # pass
        return None

    def checkForDayPortionErrors(self):
        line_errors = []
        count = 1
        choices = ['DAY', 'MORNING', 'AFTERNOON']
        for day_portion in self.lines['day_portion']:
            if day_portion not in choices:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForReasonErrors(self):
        # pass
        return None

    def checkForColorErrors(self, color_name):
        line_errors = []
        count = 1
        for color in self.lines[color_name]:
            try:
                int(color)
            except:
                line_errors.append(count)
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForRedErrors(self):
        return self.checkForColorErrors('red')

    def checkForGreenErrors(self):
        return self.checkForColorErrors('green')

    def checkForBlueErrors(self):
        return self.checkForColorErrors('blue')

    def checkForColorRangeErrors(self):
        line_errors = []
        for i in range(len(self.lines['red'])):
            red = int(self.lines['red'][i])
            green = int(self.lines['green'][i])
            blue = int(self.lines['blue'][i])

            # must be between 0 and 255
            if red < 0 or red > 255 or green < 0 or green > 255 or blue < 0 or blue > 255:
                line_errors.append(i+1)
        if len(line_errors) > 0:
            return line_errors
        return None

