# Version 0.1
#
#  VacationPlanner is copyright 2005 Jeffrey G. Pittman.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#  MA 02111-1307 USA

import csv, string, cStringIO

import os, sys, math, random

from reportlab.lib import colors
from reportlab.pdfgen.canvas import Canvas

from reportlab.graphics.shapes import Group
from reportlab.graphics.shapes import Drawing

import csv
import calendar
# the dateutil functionality should perhaps be added:
#from dateutil.relativedate import *
#from dateutil.rrule import *
#from dateutil.parser import *
from datetime import date
from datetime import datetime
from datetime import timedelta

from Products.VacationPlanner.utils import getrlcolor
from Products.VacationPlanner.utils import colorstr
from Products.VacationPlanner.utils import wordWrap
from Products.VacationPlanner.utils import rgb256

from Products.VacationPlanner.graphics import Line
from Products.VacationPlanner.graphics import DashedLine
from Products.VacationPlanner.graphics import Circle
from Products.VacationPlanner.graphics import Rectangle
from Products.VacationPlanner.graphics import IDRectangle
from Products.VacationPlanner.graphics import Text
from Products.VacationPlanner.graphics import TransformedText
from Products.VacationPlanner.graphics import AlignedText
from Products.VacationPlanner.graphics import TransformedAndAlignedText

from reportlab.graphics.shapes import Line as RLLine
from reportlab.graphics.shapes import Rect as RLRect
from reportlab.graphics.shapes import Circle as RLCircle
from reportlab.graphics.shapes import Ellipse as RLEllipse
from reportlab.graphics.shapes import PolyLine as RLPolyLine
from reportlab.graphics.charts.textlabels import Label

from reportlab.lib.colors import PCMYKColor
from reportlab.lib.colors import cmyk2rgb

backgroundGrey = PCMYKColor(20,20,20,20)    # Grey
backgroundGrey1 = PCMYKColor(3,3,3,3)       # Grey 1
backgroundGrey2 = PCMYKColor(5,5,5,5)       # Grey 2

holidays = [[2006,1,1,"New Year's Day"],
            [2006,2,2,"Groundhog Day"],
            [2006,2,14,"St. Valentine's Day"],
            [2006,2,20,"Presidents Day (3rd Monday of February)"],
            [2006,3,5,"Mother-in-Law Day"],
            [2006,3,17,"St. Patrick's Day"],
            [2006,3,20,"Vernal Equinox"],
            [2006,4,1,"April Fool's Day"],
            [2006,4,2,"Daylight Saving Time begins; clocks move forward (1st Sunday of April)"],
            [2006,4,15,"Income Tax Day"],
            [2006,4,28,"Arbor Day (varies from state to state)"],
            [2006,5,14,"Mother's Day (2nd Sunday of May)"],
            [2006,5,20,"Armed Forces Day (3rd Saturday of May)"],
            [2006,5,29,"Memorial Day (Last Monday of May)"],
            [2006,6,18,"Father's Day (3rd Sunday of June)"],
            [2006,6,21,"Summer Solstice"],
            [2006,7,4,"Independence Day"],
            [2006,9,4,"Labor Day (1st Monday of September)"],
            [2006,9,10,"Grandparents Day (2nd Sunday of September; varies from state to state)"],
            [2006,9,22,"Autumnal Equinox"],
            [2006,10,9,"Columbus Day (2nd Monday of October)"],
            [2006,10,29,"Daylight Saving Time ends; clocks move back (Last Sunday in October)"],
            [2006,10,31,"All Hallows Eve  (Halloween)"],
            [2006,11,5,"Election Day (1st Tuesday after 1st Monday for even years)"],
            [2006,11,11,"Veteran's Day"],
            [2006,11,23,"Thanksgiving Day (4th Thursday in November)"],
            [2006,12,21,"Winter Solstice"],
            [2006,12,24,"Christmas Eve"],
            [2006,12,25,"Christmas"],
            [2006,12,31,"New Year's Eve"],
            [2007,1,1,"New Year's Day"],
            [2007,2,2,"Groundhog Day"],
            [2007,2,14,"St. Valentine's Day"],
            [2007,2,19,"President's Day (3rd Monday of February)"],
            [2007,3,5,"Mother-in-Law Day"],
            [2007,3,17,"St. Patrick's Day"],
            [2007,3,20,"Vernal Equinox"],
            [2007,4,1,"April Fool's Day"],
            [2007,4,1,"Daylight Saving Time begins; clocks move forward (1st Sunday of April)"],
            [2007,4,15,"Income Tax Day"],
            [2007,4,28,"Arbor Day (varies from state to state)"],
            [2007,5,13,"Mother's Day (2nd Sunday of May)"],
            [2007,5,19,"Armed Forces Day (3rd Saturday of May)"],
            [2007,5,28,"Memorial Day (Last Monday of May)"],
            [2007,6,17,"Father's Day (3rd Sunday of June)"],
            [2007,6,21,"Summer Solstice"],
            [2007,7,4,"Independence Day"],
            [2007,9,3,"Labor Day (1st Monday of September)"],
            [2007,9,9,"Grandparent's Day (2nd Sunday of September; varies from state to state)"],
            [2007,9,22,"Autumnal Equinox"],
            [2007,10,8,"Columbus Day (2nd Monday of October)"],
            [2007,10,28,"Daylight Saving Time ends; clocks move back (Last Sunday in October)"],
            [2007,10,31,"All Hallows Eve  (Halloween)"],
            [2007,11,5,"Election Day (1st Tuesday after 1st Monday for even years)"],
            [2007,11,11,"Veterans' Day"],
            [2007,11,22,"Thanksgiving Day (4th Thursday in November)"],
            [2007,12,21,"Winter Solstice"],
            [2007,12,24,"Christmas Eve"],
            [2007,12,25,"Christmas"],
            [2007,12,31,"New Year's Eve"]]

def isHoliday(dateitem):
    for holiday in holidays:
        if dateitem.year == holiday[0] and date.month == holiday[1] and dateitem.day == holiday[2]:
            return 1
    return 0

class TimeRange:
    def __init__(self, start, end):
        self.start = start
        self.end = end

class RecurrenceInterval:
    def __init__(self, interval):
        self.interval = interval

    def isHourly(self):
        if self.interval == 'HOURLY':
            return 1
        else:
            return 0

    def isDaily(self):
        if self.interval == 'DAILY':
            return 1
        else:
            return 0

    def isWeekly(self):
        if self.interval == 'WEEKLY':
            return 1
        else:
            return 0

    def isMonthly(self):
        if self.interval == 'MONTHLY':
            return 1
        else:
            return 0

    def isYearly(self):
        if self.interval == 'YEARLY':
            return 1
        else:
            return 0

class Event:
    def __init__(self, year, month, day, hour, minute, 
                 duration, description, color):
        self.year = year
        self.month = month
        self.day = day
        self.hour = hour
        self.minute = minute
        self.duration = duration
        self.description = description
        self.color = color

        self.start = date(year, month, day)
        self.end = self.start + self.duration

class RecurringEvent:
    def __init__(self, year, month, day, hour, minute, 
                       duration, recurrence_interval, start, end,
                       description, color):
        self.year = year
        self.month = month
        self.day = day
        self.hour = hour
        self.minute = minute
        self.duration = duration
        self.recurrence_interval = RecurrenceInterval(recurrence_interval)
        self.start = start
        self.end = end
        self.description = description
        self.color = color
        self.events = self.generateEvents()

    def generateEvents(self):
        events = []
        if self.recurrence_interval.isWeekly():
            day_of_week = calendar.weekday(self.year,self.month,self.day)
            for target_weekday in self.getInstancesOfWeekday(day_of_week):
                events.append(Event(target_weekday.year, 
                                    target_weekday.month, 
                                    target_weekday.day, 
                                    self.hour, 
                                    self.minute, 
                                    self.duration,
                                    self.description,
                                    self.color))
        start_month = -1
        for year in range(self.start.year, self.end.year+1):
            if self.recurrence_interval.isYearly():
                events.append(Event(year, self.month, self.day, self.hour, 
                                    self.minute, self.duration,
                                    self.description, self.color))
            else:
                if start_month == -1:
                    start_month = self.start.month
                    if year == self.end.year:
                        end_month = self.end.month
                    else:
                        end_month = 12
                elif year == self.end.year:
                    if year == self.start.year:
                        start_month = self.start.month
                    else:
                        start_month = 1
                    end_month = self.end.month
                else:
                    start_month = 1
                    end_month = 12
                start_day = -1
                for month in range(start_month, end_month+1):
                    if self.recurrence_interval.isMonthly():
                        events.append(Event(year, month, self.day, self.hour, 
                                            self.minute, self.duration,
                                            self.description, self.color))
                    else:
                        if start_day == -1:
                            start_day = self.start.day
                            if month == self.end.month:
                                end_day = self.end.day
                            else:
                                end_day = calendar.monthrange(year, month)[1]
                        elif month == self.end.month:
                            if month == self.start.month:
                                start_day = self.start.day
                            else:
                                start_day = 1
                            end_day = self.end.day
                        else:
                            start_day = 1
                            end_day = calendar.monthrange(year, month)[1]
                        start_hour = -1
                        for day in range(start_day, end_day+1):
                            if self.recurrence_interval.isDaily():
                                events.append(Event(year, month, day, self.hour, 
                                                    self.minute,
                                                    self.duration,
                                                    self.description,
                                                    self.color))
                            elif self.recurrence_interval.isHourly():
                                if start_hour == -1:
                                    start_hour = self.start.hour
                                    if day == self.end.day:
                                        end_hour == self.end.hour
                                    else:
                                        end_hour = 24
                                elif day == self.end.day:
                                    if day == self.start.day:
                                        start_hour = self.start.hour
                                    else:
                                        start_hour = 1
                                    end_hour = self.end.hour
                                else:
                                    start_hour = 1
                                    end_hour = 24
                                start_minute = -1
                                for hour in range(start_hour, end_hour+1):
                                    events.append(Event(year, month, day, hour, 
                                                        self.minute,
                                                        self.duration,
                                                        self.description,
                                                        self.color))
                            else:
                                pass
        return events

    def getEvents(self):
        return self.events
                                    
    def getInstancesOfWeekday(self, weekday):
        weekdays = []
        day = self.start.day
        month = self.start.month
        year = self.start.year
        one_day = date(2005,1,1) - date(2004,12,31)
        day = self.start + one_day
        weekdays.append(self.start)
        while day != self.end:
            if calendar.weekday(day.year, day.month, day.day) == weekday:
                weekdays.append(day)
            day += one_day
            
        #days_in_current_month = calendar.monthrange(year, month, day)
        #for year in range(self.start.year, self.end.year+1):
        #    if year == self.start.year:
        #        for month in range(self.start.month, 13):
        #            if month == self.start.month:
        #                for day in range(self.start.day, calendar.monthrange(year, month)[1]+1):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #            elif month == self.end.month:
        #                for day in range(1, self.end.day):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #            else:
        #                for day in range(1, calendar.monthrange(year, month)[1]+1):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #    elif year == self.end.year:
        #        for month in range(1, self.end.month+1):
        #            if month == self.end.month:
        #                for day in range(1, self.day+1):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #            else:
        #                for day in range(1, calendar.monthrange(year, month)[1]+1):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #    else:
        #        for month in range(1, 13):
        #            for day in range(1, calendar.monthrange(year, month)[1]+1):
        #                if calendar.weekday(year, month, day) == weekday:
        #                    weekdays.append(date(year, month, day))
        return weekdays

class DayPortion:
    def __init__(self, portion):
        self.portion = portion

    def isMorning(self):
        if self.portion == 'MORNING':
            return 1
        else:
            return 0

    def isAfternoon(self):
        if self.portion == 'AFTERNOON':
            return 1
        else:
            return 0

    def isDay(self):
        if self.portion == 'DAY':
            return 1
        else:
            return 0

class VacationDay:
    def __init__(self, year, month, day, day_portion_off, reason, color, username):
        self.date = date(year, month, day)
        self.day_portion = DayPortion(day_portion_off)
        self.reason = reason
        self.color = color
        self.username = username

    def getStart(self):
        if self.day_portion.isDay():
            return 7
        if self.day_portion.isMorning():
            return 7
        if self.day_portion.isAfternoon():
            return 12
        return 7

    def getDuration(self):
        if self.day_portion.isDay():
            return timedelta(hours=10)
        if self.day_portion.isMorning():
            return timedelta(hours=5)
        if self.day_portion.isAfternoon():
            return timedelta(hours=5)
        return timedelta(hours=10)

class Person:
    def __init__(self, username, realname, groupname, vacation_days, color):
        self.username = username
        self.realname = realname
        self.groupname = groupname
        self.vacation_days = []
        self.addVacationDays(vacation_days)
        self.color = color

    def addVacationDay(self, year, month, day, day_portion_off, reason, color):
        self.vacation_days.append(VacationDay(year, month, day, day_portion_off, reason, color, self.username))

    def addVacationDayObject(self, vacation_day):
        self.vacation_days.append(vacation_day)

    def addVacationDays(self, vacation_days):
        for vacation_day in vacation_days:
            self.addVacationDayObject(vacation_day)

    def getVacationForDate(self, search_date):
        vacation_for_date = []
        for vacation_day in self.vacation_days:
            if vacation_day.date == search_date:
                return vacation_day
        return None

class EventContainer:
    def __init__(self):
        self.events = []
        self.recurring_events = []
        self.person_associations = {}
        self.group_associations = {}

    def addEvent(self, event):
        self.events.append(event)

    def addRecurringEvent(self, recurring_event):
        self.recurring_events.append(recurring_event)

    def getAllEvents(self):
        all_events = []
        for event in self.events:
            all_events.append(event)
        for recurring_event in self.recurring_events:
            for event in recurring_event.getEvents():
                all_events.append(event)
        return all_events

    def getAllEventsForDate(self, search_date):
        all_events = []
        for event in self.getAllEvents():
            if event.start == search_date:
                all_events.append(event)
        return all_events

    def getAllEventsForDateForPerson(self, search_date, username):
        all_events = []
        for event in self.person_associations[username]:
            if event.start == search_date:
                all_events.append(event)
        return all_events

    def getAllEventsForDateForGroup(self, search_date, group):
        all_events = []
        for event in self.person_associations[group]:
            if event.start == search_date:
                all_events.append(event)
        return all_events

    def addAssociationsForPersonToEvents(self, username, events):
        for event in events:
            if self.person_associations.has_key(username):
                self.person_associations[username].append(event)
            else:
                self.person_associations[username] = []
                self.person_associations[username].append(event)
        
    def addAssociationsForGroupToEvents(self, username, events):
        for event in events:
            if self.group_associations.has_key(username):
                self.group_associations[username].append(event)
            else:
                self.group_associations[username] = []
                self.group_associations[username].append(event)
        
    def addAssociationForPersonToRecurringEvent(self, username, recurring_event):
        self.addAssociationsForPersonToEvents(username, recurring_event.getEvents())
        
    def addAssociationForGroupToRecurringEvent(self, groupname, recurring_event):
        self.addAssociationsForPersonToEvents(groupname, recurring_event.getEvents())
        
    def getMonthNumber(self, month_name):
        month_names = { 'January':1,
                        'February':2,
                        'March':3,
                        'April':4,
                        'May':5,
                        'June':6,
                        'July':7,
                        'August':8,
                        'September':9,
                        'October':10,
                        'November':11,
                        'December':12 }

        return month_names[month_name]

    def getMonthName(self, month):
        month_names = { 1:'January',
                        2:'February',
                        3:'March',
                        4:'April',
                        5:'May',
                        6:'June',
                        7:'July',
                        8:'August',
                        9:'September',
                        10:'October',
                        11:'November',
                        12:'December' }
    
        return month_names[month]
    
    def getWeekdayAbbreviation(self, year, month, day):
        weekday_abbreviations = { 0:'MON',
                                  1:'TUE',
                                  2:'WED',
                                  3:'THU',
                                  4:'FRI',
                                  5:'SAT',
                                  6:'SUN' }

        return weekday_abbreviations[calendar.weekday(year, month, day)]

class PersonContainer:
    def __init__(self):
        self.persons = []

    def addPerson(self, person):
        self.persons.append(person)

    def getVacationsForPersons(persons, search_start, search_end):
        days_and_persons = {}
        mornings_and_persons = {}
        afternoons_and_persons = {}
        day_delta = timedelta(days=1)
        day = search_start
        while day <= search_end:
            days_and_persons[day] = []
            mornings_and_persons[day] = []
            afternoons_and_persons[day] = []
            for person in persons:
                vacation_day = person.getVacationForDate(day)
                if vacation_day is not None:
                    if vacation_day.day_portion.isDay():
                        days_and_persons[day].append(person)
                    elif vacation_day.day_portion.isMorning():
                        mornings_and_persons[day].append(person)
                    else:
                        afternoons_and_persons[day].append(person)
            day += day_delta
        return (days_and_persons, mornings_and_persons, afternoons_and_persons)

    def getAvailableDaysForPersons(day_portion, persons, search_start, search_end):
        vacation_days_and_persons, vacation_mornings_and_persons, vacation_afternoons_and_persons = self.getVacationsForPersons(persons, search_start, search_end)
        if day_portion.isDay(): 
            available_days = []
            day = search_start
            while day <= search_end:
                if len(vacation_days_and_persons[day]) == 0:
                    available_days.append(day)
            return available_days
        elif day_portion.isMorning(): 
            available_mornings = []
            day = search_start
            while day <= search_end:
                if len(vacation_mornings_and_persons[day]) == 0:
                    available_mornings.append(day)
                day += day_delta
            return available_mornings
        elif day_portion.isAfternoon(): 
            available_afternoons = []
            day = search_start
            while day <= search_end:
                if len(vacation_afternoons_and_persons[day]) == 0:
                    available_afternoons.append(day)
                day += day_delta
            return available_afternoons
        return []

    def getFirstAvailableDayForPersons(day_portion, persons, search_start, search_end):
        available_days = self.getAvailableDaysForPersons(day_portion, persons, search_start, search_end)
        if len(available_days) == 0:
            return None
        else:
            return available_days[0]

    def getBestAvailableForPersons(day_portion, persons, search_start, search_end):
        vacation_days_and_persons, vacation_mornings_and_persons, vacation_afternoons_and_persons = self.getVacationsForPersons(persons, search_start, search_end)
        search_dict = {}
        if day_portion.isDay(): 
            search_dict = vacation_days_and_persons
        elif day_portion.isMorning(): 
            search_dict = vacation_mornings_and_persons
        elif day_portion.isAfternoon(): 
            search_dict = vacation_afternoons_and_persons
        available_days = []
        day = search_start
        max_day = day
        while day <= search_end:
            if len(search_dict[day]) > len(search_dict[max_day]):
                max_day = day
        return (max_day, search_dict[max_day])

class PersonAndEventDrawing:
    def __init__(self, left_margin, right_margin, bottom_margin, top_margin, 
                 first_day, last_day, html_filename, pdf_filename):
        self.event_container = EventContainer()
        self.person_container = PersonContainer()
        self.total_width = 0
        self.total_height = 0
        self.left_margin = left_margin
        self.right_margin = right_margin
        self.bottom_margin = bottom_margin
        self.top_margin = top_margin
        self.first_day = first_day
        self.last_day = last_day
        self.html_filename = html_filename
        self.pdf_filename = pdf_filename

    def addPerson(self, person):
        self.person_container.addPerson(person)

    def addEvent(self, event):
        self.event_container.addEvent(event)

    def addRecurringEvent(self, recurring_event):
        self.event_container.addRecurringEvent(recurring_event)

    def getDrawing(self):
        pass

    def write_html(self):
        html_lines = self.getHTML() 
        if html_lines:
            htmlname = self.html_filename
            file = open(htmlname,'w')
            file.writelines(html_lines) 
            file.close()

    def getHTML(self):
        pass

    def writePDF(self):
        #pdfPath = os.path.splitext('panelcalendar.pdf')
        drawing = self.getDrawing()
        if drawing:
            c = Canvas(self.pdf_filename, pagesize=(self.total_width, self.total_height))
            c.setFont('Times-Roman', 12)
            drawing.drawOn(c, 0, 0)
            c.save()

    def getPDFData(self):
        drawing = self.getDrawing()
        if drawing:
            c = Canvas(self.pdf_filename, pagesize=(self.total_width, self.total_height))
            c.setFont('Times-Roman', 12)
            drawing.drawOn(c, 0, 0)
            return c.getpdfdata()

class VacationPlanningCalendar(PersonAndEventDrawing):
    """
    The basic structure of SVG emitting methods is a modification of:
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/325823,
    whereby the strarray() method emits a list of lines that are
    added to a growing SVG file.

    SVG help at www.carto.net was also very useful.    
    """

    def __init__(self, left_margin, right_margin, bottom_margin, top_margin, 
                 first_day, last_day, html_filename, pdf_filename,
                 panel_width, column_width, row_height):
        PersonAndEventDrawing.__init__(self, left_margin, right_margin, bottom_margin, top_margin, 
                                             first_day, last_day, html_filename, pdf_filename)
        self.panel_width = panel_width
        self.column_width = column_width
        self.row_height = row_height

        self.total_width = self.left_margin + self.panel_width + self.right_margin
    
        days = (self.last_day - self.first_day).days

        self.panel_height = days * self.row_height
        self.total_height = self.panel_height + self.bottom_margin + self.top_margin

        self.month_names_and_numbers = { 'January':1,
                                         'February':2,
                                         'March':3,
                                         'April':4,
                                         'May':5,
                                         'June':6,
                                         'July':7,
                                         'August':8,
                                         'September':9,
                                         'October':10,
                                         'November':11,
                                         'December':12 }

        self.month_numbers_and_names = { 1:'January',
                                         2:'February',
                                         3:'March',
                                         4:'April',
                                         5:'May',
                                         6:'June',
                                         7:'July',
                                         8:'August',
                                         9:'September',
                                         10:'October',
                                         11:'November',
                                         12:'December' }
    
    def yUp(self, y):
        return self.total_height - float(y)

    def getHTMLForContainingSVG(self, name):        
        var = ["<body>\n",
               "<ul>\n",
               "    <li><b>Zooming</b>: use the CTRL-Key (or Apple-Key) to drag a rectangle and zoom in.\n",
               "        <br />Use the right mouse button (CTRL-Key plus mouse click on the Mac) to get a context menu, <br />where you can choose step-by-step zooming and can go back to the original extent.</li>\n",
               "    <li><b>Zoom Out</b>: like above, but with SHIFT-Key pressed.</li>\n",
               "    <li><b>Panning</b>: Press the ALT-Key and move the mouse cursor while a hand-icon appears.</li>\n",
               "    <li><b>Anti-Aliasing</b>: May be turned on and off with the help of the context menu. (Higher Quality).</li>\n",
               "</ul>\n",
               "<h2>SVG Vacation Planner</h2>\n",
               "          <embed width=\"100%%\" height=\"660px\" src=\"%s.svg\" type=\"image/svg+xml\" />\n" % name,
               "</body>\n"]
        return var

    def getHTML(self):
        var = ["<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n",
               "<html>\n",
               "<head>\n",
               " <title>Vacation Planner, Basic Listing</title>\n",
               "</head>\n",
               "<body><a name=\"top\"></a>\n",
               "  <table>\n",
               "    <tbody>\n",
               "      <tr>\n"]

        person_width = self.panel_width / len(self.person_container.persons)
        self.panel_width = person_width * len(self.person_container.persons) + (len(self.person_container.persons) * self.column_width)
        
        day_delta = timedelta(days = 1)
    
        days = (self.last_day - self.first_day).days
    
        self.total_width = self.left_margin + self.panel_width + self.right_margin
    
        self.panel_height = days * self.row_height
        self.total_height = self.panel_height + self.bottom_margin + self.top_margin
    
        day = self.first_day
        i = 1
        while day <= self.last_day:
            var += "        <td>%d/%d</td>" % (day.month,day.day)
    
            day += day_delta
                    
            persons_for_all_day = []
            persons_for_morning = []
            persons_for_afternoon = []
            for person in self.person_container.persons:
                vacation_day = person.getVacationForDate(day)
                if vacation_day is not None:
                    portion_string = ""
                    if vacation_day.day_portion.isDay():
                        persons_for_all_day.append(person)
                    elif vacation_day.day_portion.isMorning():
                        persons_for_morning.append(person)
                    else: # afternoon
                        persons_for_afternoon.append(person)
            if len(persons_for_all_day) == 0 and len(persons_for_morning) == 0 and len(persons_for_afternoon):
                var += "<td>OPEN</td></tr>\n" 
            else:
                unavailable_string = ""
                for person in persons_for_all_day:
                    unavailable_string += "%s " % person.username
                if len(persons_for_all_day) > 0:
                    unavailable_string += "(OUT ALL DAY);"
                for person in persons_for_morning:
                    unavailable_string += "%s " % person.username
                if len(persons_for_morning) > 0:
                    unavailable_string += "(OUT MORNING);"
                for person in persons_for_afternoon:
                    unavailable_string += "%s " % person.username
                if len(persons_for_afternoon) > 0:
                    unavailable_string += "(OUT AFTERNOON);"
                var += "<td>%s</td></tr>\n" % unavailable_string
        var += "    </tbody>\n",
        var += "  </table>\n",
        var += "</body>\n",
        var += "</html>\n",
        return var

    def getOpenerSection(self):
        opener_section = \
          ["<?xml version=\"1.0\"?>\n",
           "<svg  id=\"vacationplanner\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" onload=\"Init(evt)\" onmousemove=\"GetTrueCoords(evt); ShowTooltip(evt, true)\" onmouseout=\"ShowTooltip(evt, false)\" height=\"%d\" width=\"%d\">\n" % (self.total_height,self.total_width)]
        return opener_section
                      
    def getCloserSection(self):
        closer_section = ["</svg>\n"]
        return closer_section
        
    def getVacationPlannerSection(self):
        # For SVG, the 0,0 point is in the upper left corner, as compared
        # to the lower left corner for the reportlab toolkit. We handle
        # this difference in the y-axis by use of the yUp() method.
      
        person_width = self.panel_width / len(self.person_container.persons)
        self.panel_width = person_width * len(self.person_container.persons) + (len(self.person_container.persons) * self.column_width)
        
        day_delta = timedelta(days = 1)
    
        days = (self.last_day - self.first_day).days
    
        self.total_width = self.left_margin + self.panel_width + self.right_margin
    
        self.panel_height = days * self.row_height
        self.total_height = self.panel_height + self.bottom_margin + self.top_margin
    
        # translate the whole thing?
        #g.translate(self.left_margin, self.bottom_margin)

        var = []

        day = self.first_day
        i = 1
        while day <= self.last_day:
            if calendar.weekday(day.year, day.month, day.day) not in [5,6]:
                rectangle = Rectangle((self.left_margin + 5, 
                                       self.yUp(self.bottom_margin + ((i - 1) * self.row_height) + self.row_height)), 
                                      self.panel_width,
                                      self.row_height,
                                      0.1,
                                      rgb256(cmyk2rgb((backgroundGrey2.cyan, 
                                                       backgroundGrey2.magenta,
                                                       backgroundGrey2.yellow,
                                                       backgroundGrey2.black))),
                                      rgb256(cmyk2rgb((backgroundGrey2.cyan, 
                                                       backgroundGrey2.magenta,
                                                       backgroundGrey2.yellow,
                                                       backgroundGrey2.black))))
                var += rectangle.strarray()

                line = DashedLine((self.left_margin + 5, 
                                   self.yUp(self.bottom_margin + (i * self.row_height - (self.row_height / 2)))),
                                  (self.left_margin + self.panel_width, 
                                   self.yUp(self.bottom_margin + (i * self.row_height - (self.row_height / 2)))),
                                  0.2,
                                  rgb256(colors.black.rgb()),
                                  '1,2')
                var += line.strarray()
            else:
                rectangle = Rectangle((self.left_margin + 5, 
                                       self.yUp(self.bottom_margin + ((i - 1) * self.row_height) + self.row_height)), 
                                      self.panel_width,
                                      self.row_height,
                                      0.1,
                                      rgb256(cmyk2rgb((backgroundGrey1.cyan, 
                                                       backgroundGrey1.magenta,
                                                       backgroundGrey1.yellow,
                                                       backgroundGrey1.black))),
                                      rgb256(cmyk2rgb((backgroundGrey1.cyan, 
                                                       backgroundGrey1.magenta,
                                                       backgroundGrey1.yellow,
                                                       backgroundGrey1.black))))
                var += rectangle.strarray()

            # left margin labels
            label = AlignedText((self.left_margin + 5, 
                                 self.yUp(self.bottom_margin + (i * self.row_height - (self.row_height / 2)))),
                                "%d" % (day.day),
                                rgb256(colors.black.rgb()),
                                6,
                                "end")
            var += label.strarray()

            # right margin labels
            label = AlignedText((self.left_margin + self.panel_width + 6, 
                                 self.yUp(self.bottom_margin + (i * self.row_height - (self.row_height / 2)))),
                                "%d/%d" % (day.month,day.day),
                                rgb256(colors.black.rgb()),
                                6,
                                "start")
            var += label.strarray()

            # left margin month labels
            if day.day == 15:
                label = TransformedAndAlignedText((self.left_margin-(self.left_margin/3), 
                                                   self.yUp(self.bottom_margin + (i * self.row_height - (self.row_height / 2)))),
                                                  "%s" % (self.getMonthName(day.month)),
                                                  rgb256(colors.black.rgb()),
                                                  12,
                                                  -90,
                                                  'middle')
                var += label.strarray()

            # draw the month divisions between the margin month labels
            if day.day == 1:
                # left margin
                line = Line((self.left_margin, 
                             self.yUp(self.bottom_margin + (i * self.row_height - (self.row_height / 2)) - (self.row_height/2))),
                            (self.left_margin-(self.left_margin/2), 
                             self.yUp(self.bottom_margin + ((i * self.row_height - (self.row_height / 2)) - (self.row_height/2)))),
                            0.5,
                            rgb256(colors.black.rgb()))
                var += line.strarray()
    
            # draw the month division for the top
            if day.month == 12 and day.day == 31:
                line = Line((self.left_margin, 
                             self.yUp(self.bottom_margin + (i * self.row_height - (self.row_height / 2)) + (self.row_height/2))),
                            (self.left_margin-(self.left_margin/2), 
                             self.yUp((self.bottom_margin + (i * self.row_height - (self.row_height / 2)) + (self.row_height/2)))),
                            0.5,
                            rgb256(colors.black.rgb()))
                var += line.strarray()
    
            day += day_delta
            i += 1
    
        x = 5
                    
        for person in self.person_container.persons:
            for vacation_day in person.vacation_days:
                line = DashedLine((self.left_margin + x + ((person_width+self.column_width) / 2)-1, 
                                   self.yUp(self.bottom_margin + self.row_height)), 
                                  (self.left_margin + x + ((person_width+self.column_width) / 2)-1, 
                                   self.yUp(self.bottom_margin + self.panel_height + self.row_height)),
                                  0.2,
                                  rgb256(colors.black.rgb()),
                                  '1,2')
                var += line.strarray()

                day_count = (vacation_day.date - self.first_day).days
                y = int(float(day_count) * float(self.row_height))

                # plot a rectangle for the vacation
                if vacation_day.day_portion.isDay():
                    rectangle = Rectangle((self.left_margin + x + person_width/2, 
                                           self.yUp(self.bottom_margin + (y - 1) + self.row_height)),
                                          person_width,
                                          self.row_height,
                                          0.1,
                                          rgb256(vacation_day.color.rgb()),
                                          rgb256(vacation_day.color.rgb()))
                    var += rectangle.strarray()

                elif vacation_day.day_portion.isMorning():
                    rectangle = Rectangle((self.left_margin + x + person_width/2, 
                                           self.yUp(self.bottom_margin + (y - 1) + self.row_height/2)),
                                          person_width,
                                          self.row_height / 2,
                                          0.1,
                                          rgb256(vacation_day.color.rgb()),
                                          rgb256(vacation_day.color.rgb()))
                    var += rectangle.strarray()
                else: # afternoon
                    rectangle = Rectangle((self.left_margin + x + person_width/2, 
                                           self.yUp(self.bottom_margin + (y - 1 + (self.row_height / 2)))),
                                          person_width,
                                          self.row_height / 2,
                                          0.1,
                                          rgb256(vacation_day.color.rgb()),
                                          rgb256(vacation_day.color.rgb()))
                    var += rectangle.strarray()

                # label for bottom of plot
                label = TransformedAndAlignedText(((self.left_margin + x + ((person_width + self.column_width) / 2) -1, 
                                                    self.yUp(self.bottom_margin))),
                                                  "%s" % person.username,
                                                  rgb256(colors.black.rgb()),
                                                  10,
                                                  -90,
                                                  'start')
                var += label.strarray()
    
                # label for top of plot
                label = TransformedAndAlignedText((self.left_margin + x + ((person_width + self.column_width) / 2) -1, 
                                                   self.yUp(self.bottom_margin + self.panel_height + self.row_height + 3)),
                                                  "%s" % person.username,
                                                  rgb256(colors.black.rgb()),
                                                  10,
                                                  -90,
                                                  'start')
                var += label.strarray()
            x += person_width + self.column_width 
    
        return var

    def getSVG(self):
        svg_code = self.getOpenerSection()
        svg_code += self.getVacationPlannerSection()
        svg_code += self.getCloserSection()
        return svg_code

    def getMonthNumber(self, month_name):
        return self.month_names_and_numbers[month_name]

    def getMonthName(self, month):
        return self.month_numbers_and_names[month]

    def getDrawing(self):
        '''Vacation Planning Calendar'''
    
        person_width = self.panel_width / len(self.person_container.persons)
        self.panel_width = person_width * len(self.person_container.persons) + (len(self.person_container.persons) * self.column_width)
        
        day_delta = timedelta(days = 1)
    
        days = (self.last_day - self.first_day).days
    
        self.total_width = self.left_margin + self.panel_width + self.right_margin
    
        self.panel_height = days * self.row_height
        self.total_height = self.panel_height + self.bottom_margin + self.top_margin
    
        D = Drawing(self.total_width, self.total_height)

        g = Group(transform=(1,0,0,1,0,0))
    
        g.translate(self.left_margin, self.bottom_margin)

        day = self.first_day
        i = 1
        while day <= self.last_day:
            if calendar.weekday(day.year, day.month, day.day) not in [5,6]:
                g.add(RLRect(5,
                             (i - 1) * self.row_height,
                             self.panel_width,
                             self.row_height,
                             fillColor=backgroundGrey2,
                             strokeWidth = 0.5,
                             strokeColor=backgroundGrey2))
                g.add(RLLine(5, i * self.row_height - (self.row_height / 2),
                             self.panel_width, i * self.row_height - (self.row_height / 2),
                             strokeWidth = 0.5,
                             strokeDashArray = [1, 2],
                             strokeColor = colors.black))
            else:
                g.add(RLRect(5,
                             (i - 1) * self.row_height,
                             self.panel_width,
                             self.row_height,
                             fillColor=backgroundGrey1,
                             strokeWidth = 0.5,
                             strokeColor=backgroundGrey1))

            # left margin labels
            label = Label()
            label.setOrigin(5, i * self.row_height - (self.row_height / 2))
            label.angle = 0
            label.setText("%d" % (day.day))
            label.textAnchor = 'start'
            label.boxAnchor = 'e'
            label.fontName = 'Helvetica'
            label.fontSize = 6
            g.add(label)

            # right margin labels
            label = Label()
            label.setOrigin(self.panel_width+6, i * self.row_height - (self.row_height / 2))
            label.angle = 0
            label.setText("%d/%d" % (day.month,day.day))
            label.textAnchor = 'start'
            label.boxAnchor = 'w'
            label.fontName = 'Helvetica'
            label.fontSize = 6
            g.add(label)

            # left margin month labels
            if day.day == 15:
                label = Label()
                label.setOrigin(-(self.left_margin/3), i * self.row_height - (self.row_height / 2))
                label.angle = 90
                label.setText("%s" % (self.getMonthName(day.month)))
                label.textAnchor = 'middle'
                label.boxAnchor = 'c'
                label.fontName = 'Helvetica'
                label.fontSize =12 
                g.add(label)

            # draw the month divisions between the margin month labels
            if day.day == 1:
                # left margin
                g.add(RLLine(0, (i * self.row_height - (self.row_height / 2)) - (self.row_height/2),
                             -(self.left_margin/2), (i * self.row_height - (self.row_height / 2)) - (self.row_height/2),
                             strokeWidth = 0.5,
                             strokeColor = colors.black))
    
            # draw the month division for the top
            if day.month == 12 and day.day == 31:
                g.add(RLLine(0, (i * self.row_height - (self.row_height / 2)) + (self.row_height/2),
                             -(self.left_margin/2), (i * self.row_height - (self.row_height / 2)) + (self.row_height/2),
                             strokeWidth = 0.5,
                             strokeColor = colors.black))
    
            day += day_delta
            i += 1
    
        x = 5
                    
        for person in self.person_container.persons:
            for vacation_day in person.vacation_days:
                g.add(RLLine(x + ((person_width+self.column_width) / 2)-1, self.row_height, 
                             x + ((person_width+self.column_width) / 2)-1, self.panel_height + self.row_height,
                             strokeWidth = 0.2,
                             strokeDashArray = [1, 2],
                             strokeColor = colors.lightgrey))
                day_count = (vacation_day.date - self.first_day).days
                y = int(float(day_count) * float(self.row_height))

                # plot a rectangle for the vacation
                if vacation_day.day_portion.isDay():
                    g.add(RLRect(x + person_width/2,
                                 y - 1,
                                 person_width,
                                 self.row_height,
                                 fillColor=vacation_day.color,
                                 strokeWidth = 0.5,
                                 strokeColor=vacation_day.color))
                elif vacation_day.day_portion.isMorning():
                    g.add(RLRect(x + person_width/2,
                                 y - 1,
                                 person_width,
                                 self.row_height / 2,
                                 fillColor=vacation_day.color,
                                 strokeWidth = 0.5,
                                 strokeColor=vacation_day.color))
                else: # afternoon
                    g.add(RLRect(x + person_width/2,
                                 y - 1 + (self.row_height / 2),
                                 person_width,
                                 self.row_height / 2,
                                 fillColor=vacation_day.color,
                                 strokeWidth = 0.5,
                                 strokeColor=vacation_day.color))

                # label for bottom of plot
                label = Label()
                label.setOrigin(x + ((person_width + self.column_width) / 2) -1, 0)
                label.angle = 90
                label.setText(person.username)
                label.textAnchor = 'start'
                label.boxAnchor = 'e'
                label.fontName = 'Helvetica'
                label.fontSize = 10 
                g.add(label)
    
                # label for top of plot
                label = Label()
                label.setOrigin(x + ((person_width + self.column_width) / 2) -1, 
                                self.panel_height + self.row_height + 3)
                label.angle = 90
                label.setText(person.username)
                label.textAnchor = 'start'
                label.boxAnchor = 'w'
                label.fontName = 'Helvetica'
                label.fontSize = 10 
                g.add(label)
            x += person_width + self.column_width 
    
        D.add(g)
        g = Group(transform=(1,0,0,1,0,0))
        return D


def simulateVacations():
    planning_calendar = VacationPlanningCalendar(100, 100, 100, 100, 
                                                 date(2006,1,1), date(2006,12,31),
                                                 'vacation_test.html','vacation_test.pdf',
                                                 480, 2, 10)

    # add some people
    example_people = {'smithwa':['Smith, Wanda A.','group1',color01],
                      'jonespt':['Jones, Paul Tom','group1',color02],
                      'eltontt':['Elton, Tom T.','group1',color03],
                      'rosenstockkd':['Rosenstock, Kimberly D.','group1',color04],
                      'rodriqueses':['Rodriques, Emelda S.','group1',color05],
                      'whitedc':['White, Derek C.','group1',color06],
                      'apollitanovz':['Apollitano, Vito Z.','group1',color07],
                      'edwardslc':['Edwards, Lisa C.','group2',color08],
                      'ricardojb':['Ricardo, Juan B.','group2',color09],
                      'jonrv':['Jon, Ralph V.','group2',color_green],
                      'espositorm':['Esposito, Roland M.','group2',color_blue_green],
                      'abernathygf':['Abernathy, Gloria F.','group2',color_blue],
                      'roeperav':['Roeper, Agnes V.','group2',color_violet],
                      'johnsonlk':['Johnson, Larry K.','group2',color_red_violet],
                      'bunyanct':['Bunyon, Carroll T.','group2',color_red],
                      'perryyh':['Perry, Yolanda H.','group2',color_yellow_orange],
                      'rudolfooo':['Rudolfo, Oscar O.','group2',color_dark_green],
                      'petersonhf':['Peterson, Harry F.','group3',color_burgundy],
                      'jacksonuw':['Jackson, Ulysses W.','group3',color_brown],
                      'peirotjd':['Peirot, Jacques D.','group3',color_light_blue],
                      'partridgeds':['Partridge, Donna S.','group3',color_yellow],
                      'muasa':['Mual, Saul A.','group3',color_green_grey],
                      'wittpn':['Witt, Per N.','group3',color_blue_grey],
                      'taborni':['Tabor, Nicholas I.','group3',color_light_grey],
                      'honeycuttal':['Honeycutt, Alice L.','group3',color_dark_grey]}
    persons = []
    for key in example_people.keys():
        persons.append(Person(key,example_people[key][0],example_people[key][1],[],example_people[key][2]))
    
    # give the people some vacations
    
    # vacation day reasons weighted toward the 'standard' reason (will be randomly selected)
    reasons = ['standard','standard','standard','standard','standard','standard',
               'medical','maternity','maternity','death_in_family']
    
    # random selection of durations are weighted as follows for standard vacations:
    standard_durations = [1,2,3,3,4,4,4,4,5,5,5,5,5,6,6,6,6,6,6,7,7,7,7,7,7,7,7,7,10,10,10,14,14,30]
    
    # random day portions (morning, afternoon, or the most common by far, full day)
    day_portions = ['DAY','DAY','DAY','DAY','DAY','DAY','DAY','DAY','DAY','DAY','DAY','DAY','DAY',
                     'DAY','DAY','DAY','DAY','DAY','DAY','DAY','DAY','DAY','MORNING','AFTERNOON']
    
    days_in_2006_and_2007 = []
    year = 2006
    for month in range(1, 13):
        for day in range(1, (calendar.monthrange(year, month)[1] + 1)):
            days_in_2006_and_2007.append((year, month, day))
    year = 2007
    for month in range(1, 13):
        for day in range(1, (calendar.monthrange(year, month)[1] + 1)):
            days_in_2006_and_2007.append((year, month, day))
    
    year = 2006
    
    for person in persons:
        # give each person two vacations in the year
        for i in range(2):
            month = random.randint(1,12)
            days_in_month = calendar.monthrange(year, month)[1]
            start_day = random.randint(1, days_in_month)
            day_index = 0
            index = 0
            for entry in days_in_2006_and_2007:
                if month == entry[1] and start_day == entry[2]:
                    day_index = index
                    break;
                index += 1
            reason = random.choice(reasons)
            duration = 7
            if reason == 'standard':
                duration = random.randint(1,random.choice(standard_durations))
                if duration == 1:
                    day_portion = random.choice(['MORNING','AFTERNOON','DAY'])
                else:
                    day_portion = random.choice(day_portions)
            elif reason == 'medical':
                duration = random.randint(4,30)
                day_portion = 'DAY'
            elif reason == 'maternity':
                duration = random.randrange(30,270,30)
                day_portion = 'DAY'
            elif reason == 'death_in_family':
                duration = random.randint(1,5)
                day_portion = 'DAY'
            for day_number in range(duration):
                month = days_in_2006_and_2007[day_index][1]
                day_of_month = days_in_2006_and_2007[day_index][2]
                day = date(year, month, day_of_month)
                if calendar.weekday(year, month, day_of_month) not in [5,6] and not isHoliday(day):
                    person.addVacationDay(year, month, day_of_month, day_portion, reason, getrlcolor(200, 200, 200))
                day_index += 1
        planning_calendar.addPerson(person)
    
    planning_calendar.writePDF()
    
    planning_calendar.write_html()
    
class VacationPlannerDrawing:
    def __init__(self, render_types, 
                 left_margin, right_margin, 
                 top_margin, bottom_margin, 
                 scale_to_fit, 
                 start_year, start_month, start_day, 
                 end_year, end_month, end_day, 
                 panel_width, column_width, row_height,
                 vacations_input):
        self.render_types = render_types
        self.left_margin = left_margin
        self.right_margin = right_margin
        self.top_margin = top_margin
        self.bottom_margin = bottom_margin
        self.scale_to_fit = scale_to_fit
        self.start_year = start_year
        self.start_month = start_month
        self.start_day = start_day
        self.end_year = end_year
        self.end_month = end_month
        self.end_day = end_day
        self.panel_width = panel_width
        self.column_width = column_width
        self.row_height = row_height
        self.vacations_input = vacations_input

        name = "vacationplanner"

        self.vacation_planner = VacationPlanningCalendar(self.left_margin, self.right_margin, self.top_margin, self.bottom_margin,
                                                         date(self.start_year, self.start_month, self.start_day),
                                                         date(self.end_year, self.end_month, self.end_day),
                                                         'vacation_planner_test.html','vacation_planner_test.pdf',
                                                         self.panel_width, self.column_width, self.row_height)
        usernames_and_persons = {}
        for vacation_string in self.vacations_input:
            username, realname, groupname, year, month, day, day_portion, reason, red, green, blue  = eval('[%s]' % vacation_string)
            if not usernames_and_persons.has_key(username):
                person = Person(username, realname, groupname, [], getrlcolor(red, green, blue))
                person.addVacationDay(year, month, day, day_portion, reason, getrlcolor(red, green, blue))
                usernames_and_persons[username] = person
            else:
                usernames_and_persons[username].addVacationDay(year, month, day, day_portion, reason, getrlcolor(red, green, blue))

        for username in usernames_and_persons.keys():
            self.vacation_planner.addPerson(usernames_and_persons[username])

    def getDrawing(self):
        self.drawing = self.vacation_planner.getDrawing()

        if self.scale_to_fit:
            scale_factor = float(self.height) / self.vacation_planner.total_height
            self.drawing.scale(scale_factor,scale_factor)

        return self.drawing

    def getSVG(self):
        return self.vacation_planner.getSVG()

    def getHTMLForContainingSVG(self, name):
        return self.vacation_planner.getHTMLForContainingSVG(name)

    def getHTML(self):
        return self.vacation_planner.getHTML()

    
